//@autor zhangj
//@email zhangjin0908@hotmail.com
//@date  2017年12月8日 15:56:33
var StringUtils =require('./utils/StringUtils.js')
class TxSession{
  //connection;
  //@parame session 数据库session
  constructor(session){
    this.connection = session;
  }
  //提交事物
  commit(){
    this.connection.commit();
  }
  //回滚事物
  rollback(){
    this.connection.rollback();
  }
  //关闭当前数据connection
  close(){
    this.connection.close();
  }
  /**
  *保存数据,和update以及create相比,此方法会自动判断是否存在,如果存在则修改,不存在则更新
  *@parame tablename 要更新的表的名称
  *@parame datas      要更新的数据
  */
  save(obj){
    let tablename = obj.tablename || "" ;
    let _datas;
    if(StringUtils.isArray(obj.datas)){
      _datas=obj.datas;
    }else{
      _datas=[obj.datas];
    }
    let self = this;
    let resutlt = new Array();
    for(let i =0;i<_datas.length;i++){
      resutlt.push(new Promise(function(resolve, reject){
        self.select({
            sql:"select count(1) as c  from "+tablename +" where id=:id",
            datas:{
              id:_datas[i].id
            }
          }).then(function(rd){
            let sqldata = rd.datas;
            if(sqldata[0].c==0){
              resolve(self.create({
                tablename:tablename,
                datas:_datas[i]
              }));
            }else{
              resolve(self.update({
                tablename:tablename,
                datas:_datas[i]
              }));
            }
          });
      }));
    }
    return Promise.all(resutlt,function(d){
      return d;
    });
  }
  /**
  * 更新数据
  * @parame tablename 要更新的表的名称
  * @parame datas     要更新的数据,如果是一个数组,那么会将这个数组整个进行保存
  * @return
              -rowsAffected 影响的行数
  */
  update(obj){
      let tablename  = obj.tablename || "";
      if(tablename == ""){
        throw Error("表名不能为空!")
      }
      let _datas;
      if(StringUtils.isArray(obj.datas)){
        _datas=this.conversionOfOracleSaveData(tablename,obj.datas);
      }else{
        _datas=this.conversionOfOracleSaveData(tablename,[obj.datas]);
      }

      if(_datas.length == 0){
        throw Error("要保存的数据不能为空!");
      }
      let self = this;
      return _datas.then(function(datas){
        let data = datas[0];
        let keys = new Array();
        for(let key  in data){
          keys.push(key);
        }
        var result = new Array();
        for(var i=0;i<datas.length;i++){
          result.push(new Promise(function(resolve, reject){
            let sql = "update "+tablename+" "+StringUtils.addUpdate(keys);
            var exeResult = self.connection.execute(sql,datas[i]).then(function(result){
              return result.rowsAffected;
            });
            resolve(exeResult);
          }));
        }
        return Promise.all(result,function(datas){
          return datas;
        });
      });
  }
  /**
  * 创建数据
  * @parame tablename 要创建的表的名称
  * @parame datas     要创建的数据
  * @return
              -rowsAffected 影响的行数
  */
  create(obj){
      let tablename  = obj.tablename || "";
      if(tablename == ""){
        throw Error("表名不能为空!")
      }
      let _datas;
      if(StringUtils.isArray(obj.datas)){
        _datas=this.conversionOfOracleSaveData(tablename,obj.datas);
      }else{
        _datas=this.conversionOfOracleSaveData(tablename,[obj.datas]);
      }
      if(_datas.length == 0){
        throw Error("要保存的数据不能为空!");
      }
      let self = this;
      return _datas.then(function(datas){
        let data = datas[0];
        let keys = new Array();
        for(let key  in data){
          keys.push(key);
        }
        var result = new Array();
        for(var i=0;i<datas.length;i++){
          result.push(new Promise(function(resolve, reject){
            let sql = "insert into "+tablename+" "+StringUtils.addInsert(keys);
            var exeResult = self.connection.execute(sql,datas[i]).then(function(result){
              return result.rowsAffected;
            });
            resolve(exeResult);
          }));
        }
        return Promise.all(result,function(datas){
          return datas;
        });
      });
  }
  /**
  * 删除数据
  * @parame tablename 要删除的表的名称
  * @parame datas     要删除的条件
    @return  rowsAffected 影响的行数
  */
  delete(obj){
      let tablename = obj.tablename || "";
      if(tablename == ""){
        throw Error("表名不能为空!")
      }
      let datas = obj.datas || {};

      let keys = new Array();
      for(let key  in datas){
        keys.push(key);
      }
      let where = StringUtils.addWhere(keys);
      var sql = " delete from "+ tablename + " where "+where;
      return this.connection.execute(sql,obj.datas).then(function(result){
        return result.rowsAffected;
      });
  }
  /**
  * 查询数据
  * @parame sql 要查询数据的SQL
  * @parame datas   要查询数据的参数
  * @return
  *               -datas->真实数据
                  -columns->列的信息
  */
  select(obj){
      let sql = obj.sql || "";
      if(sql == ""){
        throw Error("SQL不能为空!")
      }
      obj.datas = obj.datas || {};
      return this.connection.execute(obj.sql,obj.datas).then(function(result) {
            let metaData = result.metaData;
            let rows = result.rows;
            let re = new Array();
            for(let i =0;i<rows.length;i++){
              let data = rows[i];
              let pdata = {};
              for(let x=0;x<data.length;x++){
                pdata[metaData[x].name.toLocaleLowerCase()]=data[x];
              }
              re.push(pdata);
            }
            return {
              columns:metaData,
              datas:re
            }
      });
  }
  //获取表的信息,
  //@parame tablename 表的名称
  getTableInfo(tablename){
    tablename = tablename || "";
    tablename = tablename.toUpperCase();
    return this.select({
      sql:"SELECT * FROM user_col_comments WHERE table_name = :table_name",
      datas:{
        table_name:tablename
      }
    }).then(function(result){
      return result.datas;
    });
  }
  //将数据转换为数据库中符合数据格式的数据
  conversionOfOracleSaveData(tablename,datas){
     return this.getTableInfo(tablename).then(function(tdatas){
       let datasArray = new Array();
       for(let i=0;i<tdatas.length;i++){
         let rowData = tdatas[i];
         datasArray.push(rowData.column_name);
       }
       let resultData = new Array();
       for(let i=0;i<datas.length;i++){
         let rowData = datas[i];
         let newRowData = {};
         for(let key in rowData){
           if(datasArray.indexOf(key.toLocaleUpperCase()) == -1){
             continue;
           }else{
             newRowData[key] = rowData[key];
           }
         }
         resultData.push(newRowData);
       }
       return resultData;
    });
  }

}
module.exports = TxSession;
